"use strict";
// Thanks to https://github.com/vercel/next.js/blob/574fe0b582d5cc1b13663121fd47a3d82deaaa17/packages/next/build/babel/plugins/next-ssg-transform.ts
// Modified heavily
// !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
//
// !!!!!!!! Someone please help me remove the `@ts-ignore` in this file !!!!!!!!!!!
//
// !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
Object.defineProperty(exports, "__esModule", { value: true });
const EXPORT_PRELOAD = `preload`;
const EXPORT_STATIC_PRELOAD = `staticPreload`;
const EXPORT_GET_STATIC_PATHS = `getStaticPaths`;
const serverOnlyExportNames = new Set([
    EXPORT_PRELOAD,
    EXPORT_STATIC_PRELOAD,
    EXPORT_GET_STATIC_PATHS,
]);
function getIdentifier(path) {
    const parentPath = path.parentPath;
    if (parentPath.type === 'VariableDeclarator') {
        const pp = parentPath;
        const name = pp.get('id');
        return name.node.type === 'Identifier'
            ? name
            : null;
    }
    if (parentPath.type === 'AssignmentExpression') {
        const pp = parentPath;
        const name = pp.get('left');
        return name.node.type === 'Identifier'
            ? name
            : null;
    }
    if (path.node.type === 'ArrowFunctionExpression') {
        return null;
    }
    return path.node.id && path.node.id.type === 'Identifier'
        ? path.get('id')
        : null;
}
function isIdentifierReferenced(ident) {
    const b = ident.scope.getBinding(ident.node.name);
    if (b === null || b === void 0 ? void 0 : b.referenced) {
        // Functions can reference themselves, so we need to check if there's a
        // binding outside the function scope or not.
        if (b.path.type === 'FunctionDeclaration') {
            return !b.constantViolations
                .concat(b.referencePaths)
                // Check that every reference is contained within the function:
                .every((ref) => ref.findParent((p) => p === b.path));
        }
        return true;
    }
    return false;
}
const createMarkFunction = (state) => function markFunction(path) {
    const ident = getIdentifier(path);
    if ((ident === null || ident === void 0 ? void 0 : ident.node) && isIdentifierReferenced(ident)) {
        state.refs.add(ident);
    }
};
const createMarkImport = (state) => function markImport(path) {
    const local = path.get('local');
    if (isIdentifierReferenced(local)) {
        state.refs.add(local);
    }
};
function pageExportsTransforms({ types: t, }) {
    return {
        visitor: {
            Program: {
                enter(path, state) {
                    state.refs = new Set();
                    state.serverOnlyExports = new Set();
                    const markImport = createMarkImport(state);
                    const markFunction = createMarkFunction(state);
                    path.traverse({
                        // No idea why the second argument is always undefine
                        // It should have been `state`
                        VariableDeclarator(variablePath) {
                            if (variablePath.node.id.type === 'Identifier') {
                                const local = variablePath.get('id');
                                if (isIdentifierReferenced(local)) {
                                    state.refs.add(local);
                                }
                            }
                            else if (variablePath.node.id.type === 'ObjectPattern') {
                                const pattern = variablePath.get('id');
                                const properties = pattern.get('properties');
                                properties.forEach((p) => {
                                    const local = p.get(p.node.type === 'ObjectProperty'
                                        ? 'value'
                                        : p.node.type === 'RestElement'
                                            ? 'argument'
                                            : (function () {
                                                throw new Error('invariant');
                                            })());
                                    if (isIdentifierReferenced(local)) {
                                        state.refs.add(local);
                                    }
                                });
                            }
                            else if (variablePath.node.id.type === 'ArrayPattern') {
                                const pattern = variablePath.get('id');
                                const elements = pattern.get('elements');
                                elements.forEach((e) => {
                                    var _a, _b;
                                    let local;
                                    if (((_a = e.node) === null || _a === void 0 ? void 0 : _a.type) === 'Identifier') {
                                        local = e;
                                    }
                                    else if (((_b = e.node) === null || _b === void 0 ? void 0 : _b.type) === 'RestElement') {
                                        local = e.get('argument');
                                    }
                                    else {
                                        return;
                                    }
                                    if (isIdentifierReferenced(local)) {
                                        state.refs.add(local);
                                    }
                                });
                            }
                        },
                        // @ts-ignore
                        FunctionDeclaration: markFunction,
                        // @ts-ignore
                        FunctionExpression: markFunction,
                        // @ts-ignore
                        ArrowFunctionExpression: markFunction,
                        // @ts-ignore
                        ImportSpecifier: markImport,
                        // @ts-ignore
                        ImportDefaultSpecifier: markImport,
                        // @ts-ignore
                        ImportNamespaceSpecifier: markImport,
                        ExportNamedDeclaration(path) {
                            const insertIndicator = (path, exportName) => {
                                path.insertBefore(t.exportNamedDeclaration(t.variableDeclaration('var', [
                                    t.variableDeclarator(t.identifier(exportName), t.numericLiteral(1)),
                                ])));
                            };
                            let shouldRemove = false;
                            // Handle re-exports: export { preload } from './foo'
                            path.node.specifiers = path.node.specifiers.filter((spec) => {
                                // @ts-ignore
                                const { name } = spec.exported;
                                for (const serverOnlyExportName of serverOnlyExportNames) {
                                    if (name === serverOnlyExportName) {
                                        insertIndicator(path, serverOnlyExportName);
                                        state.serverOnlyExports.add(serverOnlyExportName);
                                        return false;
                                    }
                                }
                                return true;
                            });
                            const { declaration } = path.node;
                            // When none of Re-exports left, remove the path
                            if (!declaration && path.node.specifiers.length === 0) {
                                shouldRemove = true;
                            }
                            if (declaration && declaration.type === 'VariableDeclaration') {
                                declaration.declarations = declaration.declarations.filter((declarator) => {
                                    var _a;
                                    for (const name of serverOnlyExportNames) {
                                        if (declarator.id.name ===
                                            name &&
                                            ((_a = declarator.init) === null || _a === void 0 ? void 0 : _a.type.includes('Function')) // ArrowFunctionExpression or FunctionExpression
                                        ) {
                                            insertIndicator(path, name);
                                            state.serverOnlyExports.add(name);
                                            return false;
                                        }
                                    }
                                    return true;
                                });
                                if (declaration.declarations.length === 0) {
                                    shouldRemove = true;
                                }
                            }
                            if (declaration && declaration.type === 'FunctionDeclaration') {
                                for (const name of serverOnlyExportNames) {
                                    // @ts-ignore
                                    if (declaration.id.name === name) {
                                        shouldRemove = true;
                                        state.serverOnlyExports.add(name);
                                        insertIndicator(path, name);
                                    }
                                }
                            }
                            if (shouldRemove) {
                                path.remove();
                            }
                        },
                    });
                    if (state.serverOnlyExports.size === 0) {
                        // No server-spcific exports found
                        // No need to clean unused references then
                        return;
                    }
                    if (state.serverOnlyExports.has(EXPORT_PRELOAD) &&
                        state.serverOnlyExports.has(EXPORT_STATIC_PRELOAD)) {
                        throw new Error(`You can't use ${EXPORT_PRELOAD} and ${EXPORT_STATIC_PRELOAD} on the same page, use ${EXPORT_STATIC_PRELOAD} if you want to pre-render the page at build time, otherwise use ${EXPORT_PRELOAD}`);
                    }
                    const refs = state.refs;
                    let count;
                    function sweepFunction(path) {
                        const ident = getIdentifier(path);
                        if ((ident === null || ident === void 0 ? void 0 : ident.node) &&
                            refs.has(ident) &&
                            !isIdentifierReferenced(ident)) {
                            ++count;
                            if (t.isAssignmentExpression(path.parentPath) ||
                                t.isVariableDeclarator(path.parentPath)) {
                                path.parentPath.remove();
                            }
                            else {
                                path.remove();
                            }
                        }
                    }
                    function sweepImport(path) {
                        const local = path.get('local');
                        if (refs.has(local) && !isIdentifierReferenced(local)) {
                            ++count;
                            path.remove();
                            if (path.parent.specifiers
                                .length === 0) {
                                path.parentPath.remove();
                            }
                        }
                    }
                    // Traverse again to remove unused dependencies
                    // We do this at least once
                    // If something is removed `count` will be true so it will run again
                    // Otherwise it exists the loop
                    do {
                        ;
                        path.scope.crawl();
                        count = 0;
                        path.traverse({
                            VariableDeclarator(variablePath) {
                                if (variablePath.node.id.type === 'Identifier') {
                                    const local = variablePath.get('id');
                                    if (refs.has(local) && !isIdentifierReferenced(local)) {
                                        ++count;
                                        variablePath.remove();
                                    }
                                }
                                else if (variablePath.node.id.type === 'ObjectPattern') {
                                    const pattern = variablePath.get('id');
                                    const beforeCount = count;
                                    const properties = pattern.get('properties');
                                    properties.forEach((p) => {
                                        const local = p.get(p.node.type === 'ObjectProperty'
                                            ? 'value'
                                            : p.node.type === 'RestElement'
                                                ? 'argument'
                                                : (function () {
                                                    throw new Error('invariant');
                                                })());
                                        if (refs.has(local) && !isIdentifierReferenced(local)) {
                                            ++count;
                                            p.remove();
                                        }
                                    });
                                    if (beforeCount !== count &&
                                        pattern.get('properties').length < 1) {
                                        variablePath.remove();
                                    }
                                }
                                else if (variablePath.node.id.type === 'ArrayPattern') {
                                    const pattern = variablePath.get('id');
                                    const beforeCount = count;
                                    const elements = pattern.get('elements');
                                    elements.forEach((e) => {
                                        var _a, _b;
                                        let local;
                                        if (((_a = e.node) === null || _a === void 0 ? void 0 : _a.type) === 'Identifier') {
                                            local = e;
                                        }
                                        else if (((_b = e.node) === null || _b === void 0 ? void 0 : _b.type) === 'RestElement') {
                                            local = e.get('argument');
                                        }
                                        else {
                                            return;
                                        }
                                        if (refs.has(local) && !isIdentifierReferenced(local)) {
                                            ++count;
                                            e.remove();
                                        }
                                    });
                                    if (beforeCount !== count &&
                                        pattern.get('elements').length < 1) {
                                        variablePath.remove();
                                    }
                                }
                            },
                            // @ts-ignore
                            FunctionDeclaration: sweepFunction,
                            // @ts-ignore
                            FunctionExpression: sweepFunction,
                            // @ts-ignore
                            ArrowFunctionExpression: sweepFunction,
                            // @ts-ignore
                            ImportSpecifier: sweepImport,
                            // @ts-ignore
                            ImportDefaultSpecifier: sweepImport,
                            // @ts-ignore
                            ImportNamespaceSpecifier: sweepImport,
                        });
                    } while (count);
                },
            },
        },
    };
}
exports.default = pageExportsTransforms;
