"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.buildStandalone = exports.build = void 0;
const esbuild_1 = require("esbuild");
const consola_1 = __importDefault(require("consola"));
const chalk_1 = __importDefault(require("chalk"));
const get_vite_config_1 = require("./vite/get-vite-config");
const vite_1 = require("vite");
async function build(api) {
    const serverConfig = get_vite_config_1.getViteConfig(api, true);
    await Promise.all([
        vite_1.build(serverConfig),
        vite_1.build(get_vite_config_1.getViteConfig(api, false)),
    ]);
}
exports.build = build;
async function buildStandalone(api) {
    const inputFile = api.resolveDotReam('meta/server-context.js');
    const outFile = api.resolveDotReam('meta/server-context-bundle.js');
    const { warnings } = await esbuild_1.build({
        entryPoints: [inputFile],
        outfile: outFile,
        bundle: true,
        format: 'cjs',
        platform: 'node',
        minify: !process.env.DEBUG,
        logLevel: 'error',
    });
    for (const warning of warnings) {
        if (!warning.text.includes('Indirect calls to "require" will not be bundled')) {
            consola_1.default.warn(chalk_1.default.bold(warning.text));
            if (warning.location) {
                console.log(`${warning.location.file}:${warning.location.line}:${warning.location.column}`);
                console.log(warning.location.lineText);
            }
        }
    }
}
exports.buildStandalone = buildStandalone;
