"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const cac_1 = require("cac");
const http_1 = require("http");
const path_1 = __importDefault(require("path"));
const cli = cac_1.cac();
cli
    .command('[dir]', 'Serve a directory in dev mode', {
    ignoreOptionDefaultValue: true,
})
    .alias('dev')
    .option('--host <host>', 'Server host (default: localhost)')
    .option('--port <port>', 'Server port (default: 3000)')
    .action(handleError(async (rootDir, options) => {
    const { Ream } = await Promise.resolve().then(() => __importStar(require('./')));
    const host = options.host || 'localhost';
    const port = options.port || 3000;
    const app = new Ream({
        rootDir,
        dev: true,
    });
    const handler = await app.getRequestHandler();
    const server = http_1.createServer(handler);
    process.env.PORT = `${port}`;
    server.listen(port, host);
    console.log(`> http://${host}:${port}`);
}));
cli
    .command('build [dir]', 'Build a directory for production', {
    ignoreOptionDefaultValue: true,
})
    .option('--mode <mode>', 'Set a custom mode for your app')
    .option('--standalone', 'Bundle external dependencies in server code')
    .action(handleError(async (rootDir, flags) => {
    const { Ream } = await Promise.resolve().then(() => __importStar(require('./')));
    const app = new Ream({
        rootDir,
        mode: flags.mode,
        dev: false,
    });
    await app.build({ standalone: flags.standalone });
}));
cli
    .command('export [dir]', 'Export a hybrid site to a static site')
    .option('--mode <mode>', 'Set a custom mode for your app')
    .action(handleError(async (rootDir, flags) => {
    const { Ream } = await Promise.resolve().then(() => __importStar(require('./')));
    const app = new Ream({
        rootDir,
        mode: flags.mode,
        dev: false,
    });
    await app.build({ fullyExport: true }).catch(handleError);
}));
cli
    .command('start [dir]', 'Start a production server')
    .option('--host <host>', 'Server host (default: localhost)')
    .option('--port <port>', 'Server port (default: 3000)')
    .action(handleError(async (rootDir = '.', options) => {
    const { start } = await Promise.resolve().then(() => __importStar(require('@ream/server')));
    const { serverContext } = require(path_1.default.resolve(rootDir, '.ream/meta/server-context'));
    await start(rootDir, {
        host: options.host,
        port: options.port,
        context: serverContext,
    });
}));
cli.version(require('../package').version);
cli.help();
cli.parse();
function handleError(fn) {
    return async (...args) => {
        try {
            await fn(...args);
        }
        catch (error) {
            require('consola').error(error);
            process.exitCode = 1;
        }
    };
}
