/// <reference types="node" />
import type { SetRequired } from 'type-fest';
import { ViteDevServer, UserConfig as ViteConfig } from 'vite';
import { ReamPlugin } from './types';
import { State } from './state';
export interface Options {
    rootDir?: string;
    srcDir?: string;
    dev?: boolean;
    mode?: string;
}
export declare type Route = {
    name?: string;
    path: string;
    file: string;
    isServerRoute: boolean;
    children?: Route[];
};
export declare type ReamConfig = {
    env?: Record<string, string>;
    plugins?: Array<ReamPlugin>;
    imports?: string[];
    hmr?: {
        host?: string;
        port?: number;
    };
    vue?: {
        runtimeTemplateCompiler?: boolean;
    };
    vite?: (viteConfig: ViteConfig, opts: {
        dev: boolean;
        ssr?: boolean;
    }) => void;
    routes?: (defaultRoutes: Route[]) => Promise<Route[]> | Route[];
};
export declare const defineReamConfig: (config: ReamConfig) => ReamConfig;
export declare class Ream {
    state: State;
    rootDir: string;
    srcDir: string;
    isDev: boolean;
    inlineConfig: ReamConfig;
    config: SetRequired<ReamConfig, 'env' | 'plugins' | 'imports'>;
    configPath?: string;
    viteDevServer?: ViteDevServer;
    userEnv: Record<string, string>;
    mode: string;
    constructor(options?: Options, inlineConfig?: ReamConfig);
    resolveRootDir(...args: string[]): string;
    resolveSrcDir(...args: string[]): string;
    resolveDotReam(...args: string[]): string;
    resolveOwnDir(...args: string[]): string;
    resolveInPackage(pkg: string, target: string): string;
    loadConfig(): Promise<void>;
    get env(): Record<string, string>;
    get constants(): Record<string, string>;
    prepare({ shouldCleanDir, shouldPrepreFiles, }: {
        shouldCleanDir: boolean;
        shouldPrepreFiles: boolean;
    }): Promise<void>;
    localResolve(name: string): string | undefined;
    localRequire(name: string): any;
    getRequestHandler(): Promise<(req: import("http").IncomingMessage, res: import("http").ServerResponse) => Promise<void>>;
    build({ fullyExport, standalone, }: {
        fullyExport?: boolean;
        standalone?: boolean;
    }): Promise<void>;
}
export * from './types';
