"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __exportStar = (this && this.__exportStar) || function(m, exports) {
    for (var p in m) if (p !== "default" && !Object.prototype.hasOwnProperty.call(exports, p)) __createBinding(exports, m, p);
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Ream = exports.defineReamConfig = void 0;
const path_1 = require("path");
const vite_1 = require("vite");
const resolve_from_1 = __importDefault(require("resolve-from"));
const consola_1 = __importDefault(require("consola"));
const load_config_1 = require("./utils/load-config");
const load_plugins_1 = require("./load-plugins");
const fs_extra_1 = require("fs-extra");
const constants_1 = require("./utils/constants");
const state_1 = require("./state");
const defineReamConfig = (config) => config;
exports.defineReamConfig = defineReamConfig;
class Ream {
    constructor(options = {}, inlineConfig = {}) {
        this.inlineConfig = inlineConfig;
        this.rootDir = path_1.resolve(options.rootDir || '.');
        if (options.srcDir) {
            this.srcDir = path_1.join(this.rootDir, options.srcDir);
        }
        else {
            const hasPagesInSrc = fs_extra_1.existsSync(path_1.join(this.rootDir, 'src/pages'));
            this.srcDir = hasPagesInSrc ? path_1.join(this.rootDir, 'src') : this.rootDir;
        }
        this.isDev = Boolean(options.dev);
        this.mode = options.mode || (this.isDev ? 'development' : 'production');
        if (!process.env.NODE_ENV) {
            process.env.NODE_ENV = this.isDev ? 'development' : 'production';
        }
    }
    resolveRootDir(...args) {
        return path_1.resolve(this.rootDir, ...args);
    }
    resolveSrcDir(...args) {
        return path_1.resolve(this.srcDir, ...args);
    }
    resolveDotReam(...args) {
        return this.resolveRootDir('.ream', ...args);
    }
    resolveOwnDir(...args) {
        return path_1.resolve(constants_1.OWN_DIR, ...args);
    }
    resolveInPackage(pkg, target) {
        const pkgDir = path_1.dirname(resolve_from_1.default(process.cwd(), `${pkg}/package.json`));
        return resolve_from_1.default(pkgDir, target);
    }
    async loadConfig() {
        const { data: projectConfig = {}, path: configPath } = await load_config_1.loadConfig(this.rootDir);
        if (configPath) {
            consola_1.default.info(`Using config file: ${path_1.relative(process.cwd(), configPath)}`);
        }
        this.configPath = configPath;
        this.config = {
            ...this.inlineConfig,
            ...projectConfig,
            env: {
                ...projectConfig.env,
                ...this.inlineConfig.env,
            },
            plugins: [
                ...(this.inlineConfig.plugins || []),
                ...(projectConfig.plugins || []),
            ],
            imports: projectConfig.imports || [],
        };
    }
    get env() {
        return {
            ...this.userEnv,
            ...this.config.env,
            REAM_SOURCE_DIR: this.resolveSrcDir(),
            REAM_ROOT_DIR: this.resolveRootDir(),
        };
    }
    get constants() {
        const { env } = this;
        return {
            ...this.state.constants,
            ...Object.keys(env).reduce((res, key) => {
                const value = JSON.stringify(env[key]);
                return {
                    ...res,
                    [`import.meta.env.${key}`]: value,
                    [`process.env.${key}`]: value,
                };
            }, {}),
        };
    }
    async prepare({ shouldCleanDir, shouldPrepreFiles, }) {
        this.state = state_1.getInitialState();
        await this.loadConfig();
        this.userEnv = vite_1.loadEnv(this.mode, this.rootDir, 'REAM_');
        await load_plugins_1.loadPlugins(this);
        if (shouldCleanDir) {
            // Remove everything but cache
            await Promise.all(['templates', 'manifest', 'server', 'client', 'export', 'meta'].map((name) => {
                return fs_extra_1.remove(this.resolveDotReam(name));
            }));
        }
        if (shouldPrepreFiles) {
            consola_1.default.info('Preparing Ream files');
            const { prepareFiles } = await Promise.resolve().then(() => __importStar(require('./prepare-files')));
            await prepareFiles(this);
        }
        if (this.isDev) {
            // Create Vite dev server
            const { createServer: createViteServer } = await Promise.resolve().then(() => __importStar(require('vite')));
            const { getViteConfig } = await Promise.resolve().then(() => __importStar(require('./vite/get-vite-config')));
            const viteConfig = getViteConfig(this);
            const viteDevServer = await createViteServer(viteConfig);
            this.viteDevServer = viteDevServer;
            // Reuse Vite watcher to register `onFileChange` callbacks
            for (const callback of this.state.callbacks.onFileChange) {
                viteDevServer.watcher.on('all', callback.callback);
            }
            // Restart Ream when config file changes
            viteDevServer.watcher.on('all', async (_, file) => {
                const files = load_config_1.SUPPORTED_CONFIG_FILES.map((name) => this.resolveRootDir(name));
                if (file === this.configPath || files.includes(file)) {
                    consola_1.default.info(`Restarting Ream due to changes in ${file}`);
                    await viteDevServer.close();
                    await this.prepare({ shouldCleanDir, shouldPrepreFiles });
                }
            });
        }
    }
    localResolve(name) {
        return resolve_from_1.default.silent(this.rootDir, name);
    }
    localRequire(name) {
        const path = this.localResolve(name);
        return path && require(path);
    }
    async getRequestHandler() {
        await this.prepare({
            shouldCleanDir: this.isDev,
            shouldPrepreFiles: this.isDev,
        });
        const { getRequestHandler } = await Promise.resolve().then(() => __importStar(require('./server')));
        const handler = await getRequestHandler(this);
        return handler;
    }
    async build({ fullyExport, standalone, }) {
        await this.prepare({ shouldCleanDir: true, shouldPrepreFiles: true });
        const { build, buildStandalone } = await Promise.resolve().then(() => __importStar(require('./build')));
        await build(this);
        // Export static pages
        const { exportSite } = await Promise.resolve().then(() => __importStar(require('./export')));
        await exportSite(this.resolveDotReam(), fullyExport);
        if (standalone) {
            await buildStandalone(this);
        }
    }
}
exports.Ream = Ream;
__exportStar(require("./types"), exports);
