"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.PluginContext = void 0;
const normalize_path_1 = require("./utils/normalize-path");
class PluginContext {
    constructor(api, pluginName) {
        this.api = api;
        this.pluginName = pluginName;
    }
    get state() {
        return this.api.state;
    }
    get env() {
        return this.api.env;
    }
    get constants() {
        return this.api.constants;
    }
    ensureEnv(name, defaultValue) {
        const value = this.env[name];
        if (!value && !defaultValue) {
            throw new Error(`Plugin "${this.pluginName}" requires environment variable "${name}" to be set.`);
        }
        return value || defaultValue;
    }
    get pluginsFiles() {
        return this.state.pluginsFiles;
    }
    resolveSrcDir(...args) {
        return this.api.resolveSrcDir(...args);
    }
    resolveRootDir(...args) {
        return this.api.resolveRootDir(...args);
    }
    resolveDotReam(...args) {
        return this.api.resolveDotReam(...args);
    }
    defineConstant(name, value) {
        this.state.constants[name] = JSON.stringify(value);
    }
    addPluginFile(type, file) {
        this.state.pluginsFiles[type].add(normalize_path_1.normalizePath(file));
    }
    onPrepareFiles(callback) {
        this.state.callbacks.onPrepareFiles.add({
            pluginName: this.pluginName,
            callback,
        });
    }
    onFileChange(callback) {
        this.state.callbacks.onFileChange.add({
            pluginName: this.pluginName,
            callback,
        });
    }
}
exports.PluginContext = PluginContext;
