"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.loadConfig = exports.SUPPORTED_CONFIG_FILES = void 0;
const path_1 = __importDefault(require("path"));
const joycon_1 = __importDefault(require("joycon"));
const fs_extra_1 = __importDefault(require("fs-extra"));
const module_1 = require("module");
const esbuild_1 = require("esbuild");
const joycon = new joycon_1.default();
const transpileAndRequire = (code, filepath, isTS) => {
    const result = esbuild_1.transformSync(code, {
        sourcefile: filepath,
        loader: isTS ? 'ts' : 'js',
        target: 'es2019',
        format: 'cjs',
    });
    const m = requireFromString(result.code, filepath);
    return m.default || m;
};
joycon.addLoader({
    test: /\.[jt]s$/,
    async load(filepath) {
        const code = await fs_extra_1.default.readFile(filepath, 'utf8');
        return transpileAndRequire(code, filepath, filepath.endsWith('.ts'));
    },
});
exports.SUPPORTED_CONFIG_FILES = ['ream.config.js', 'ream.config.ts'];
function loadConfig(cwd) {
    joycon.clearCache();
    return joycon.load(exports.SUPPORTED_CONFIG_FILES, cwd);
}
exports.loadConfig = loadConfig;
// https://github.com/floatdrop/require-from-string/blob/master/index.js
function requireFromString(code, filename, opts) {
    opts = opts || {};
    const appendPaths = opts.appendPaths || [];
    const prependPaths = opts.prependPaths || [];
    if (typeof code !== 'string') {
        throw new Error('code must be a string, not ' + typeof code);
    }
    // @ts-ignore
    const paths = module_1.Module._nodeModulePaths(path_1.default.dirname(filename));
    const parent = module.parent || undefined;
    const m = new module_1.Module(filename, parent);
    m.filename = filename;
    m.paths = [...prependPaths, ...paths, ...appendPaths];
    // @ts-ignore
    m._compile(code, filename);
    const exports = m.exports;
    parent &&
        parent.children &&
        parent.children.splice(parent.children.indexOf(m), 1);
    return exports;
}
